==> gitlab-ce-v8.5.9.tar: gitlab-ce-v8.5.9/app/services/merge_requests/merge_when_build_succeeds_service.rb <==
module MergeRequests
  class MergeWhenBuildSucceedsService < MergeRequests::BaseService
    # Marks the passed `merge_request` to be merged when the build succeeds or
    # updates the params for the automatic merge
    def execute(merge_request)
      merge_request.merge_params.merge!(params)

      # The service is also called when the merge params are updated.
      already_approved = merge_request.merge_when_build_succeeds?

      unless already_approved
        merge_request.merge_when_build_succeeds = true
        merge_request.merge_user                = @current_user

        SystemNoteService.merge_when_build_succeeds(merge_request, @project, @current_user, merge_request.last_commit)
      end

      merge_request.save
    end

    # Triggers the automatic merge of merge_request once the build succeeds
    def trigger(commit_status)
      merge_requests = merge_request_from(commit_status)

      merge_requests.each do |merge_request|
        next unless merge_request.merge_when_build_succeeds?

        if merge_request.ci_commit && merge_request.ci_commit.success? && merge_request.mergeable?
          MergeWorker.perform_async(merge_request.id, merge_request.merge_user_id, merge_request.merge_params)
        end
      end
    end

    # Cancels the automatic merge
    def cancel(merge_request)
      if merge_request.merge_when_build_succeeds? && merge_request.open?
        merge_request.reset_merge_when_build_succeeds
        SystemNoteService.cancel_merge_when_build_succeeds(merge_request, @project, @current_user)

        success
      else
        error("Can't cancel the automatic merge", 406)
      end
    end

    private

    def merge_request_from(commit_status)
      branches = commit_status.ref

      # This is for ref-less builds
      branches ||= @project.repository.branch_names_contains(commit_status.sha)

      merge_requests = @project.origin_merge_requests.opened.where(source_branch: branches).to_a
      merge_requests += @project.fork_merge_requests.opened.where(source_branch: branches).to_a

      merge_requests.uniq.select(&:source_project)
    end
  end
end

==> gitlab-ce-v8.13.3.tar: gitlab-ce-v8.13.3/app/services/merge_requests/merge_when_build_succeeds_service.rb <==
module MergeRequests
  class MergeWhenBuildSucceedsService < MergeRequests::BaseService
    # Marks the passed `merge_request` to be merged when the build succeeds or
    # updates the params for the automatic merge
    def execute(merge_request)
      merge_request.merge_params.merge!(params)

      # The service is also called when the merge params are updated.
      already_approved = merge_request.merge_when_build_succeeds?

      unless already_approved
        merge_request.merge_when_build_succeeds = true
        merge_request.merge_user                = @current_user

        SystemNoteService.merge_when_build_succeeds(merge_request, @project, @current_user, merge_request.diff_head_commit)
      end

      merge_request.save
    end

    # Triggers the automatic merge of merge_request once the pipeline succeeds
    def trigger(pipeline)
      return unless pipeline.success?

      pipeline_merge_requests(pipeline) do |merge_request|
        next unless merge_request.merge_when_build_succeeds?
        next unless merge_request.mergeable?

        MergeWorker.perform_async(merge_request.id, merge_request.merge_user_id, merge_request.merge_params)
      end
    end

    # Cancels the automatic merge
    def cancel(merge_request)
      if merge_request.merge_when_build_succeeds? && merge_request.open?
        merge_request.reset_merge_when_build_succeeds
        SystemNoteService.cancel_merge_when_build_succeeds(merge_request, @project, @current_user)

        success
      else
        error("Can't cancel the automatic merge", 406)
      end
    end
  end
end
